################
# R call for the F procedures
# functional depth computation
# Stanislav Nagy
# stanislav.nagy@kuleuven.be
# 31/01/2017
#             
# Nagy, Gijbels, Hlubinka: Depth-Based Recognition of Shape Outlying Functions
# Nagy, Ferraty: Data Depth for Noisy Random Functions
################

if(FALSE){
 # to build the package from the source
 # place all the necessary source files, i.e.
 # 	- depth.fd.f
 # 	- depth.fd.R
 # 	- depth.fd-package.Rd
 # 	- DESCRIPTION
 # 	- population.rda
 # 	- population2010.rda
 # into the folder (R working directory, see getwd())/package_depth.fd
 # and run the following code
 # to build the package in Windows it is necessary to have installed Rtools,
 # see https://cran.r-project.org/bin/windows/Rtools/
 #
 setwd("package_depth.fd")
 unlink("depth.fd",recursive=TRUE)
 detach("package:depth.fd", unload=TRUE)
 remove.packages("depth.fd")
 library(roxygen2)
 library(devtools)
 package.skeleton("depth.fd",code_files="depth.fd.R")
 roxygenize("depth.fd")
 dir.create("depth.fd/src")
 file.copy("depth.fd.f","depth.fd/src/depth.fd.f")
 dir.create("depth.fd/data")
 file.copy("population.rda","depth.fd/data/population.rda")
 file.copy("population2010.rda","depth.fd/data/population2010.rda")
 file.copy("DESCRIPTION","depth.fd/DESCRIPTION",overwrite=TRUE)
 file.copy("depth.fd-package.Rd","depth.fd/man/depth.fd-package.Rd",overwrite=TRUE)
 build("depth.fd")
 install.packages("depth.fd_1.0.tar.gz", type="source", repos=NULL)
 library(depth.fd)
 help(depth.fd)
 #
 # to install the package from the "depth.fd_1.0.tar.gz" file, 
 # put this archive to the R working directory (getwd()), and run
 # install.packages("depth.fd_1.0.tar.gz", type="source", repos=NULL)
 }

#' @useDynLib depth.fd
#' @export FKS
#' @export shape.fd.analysis
#' @export shape.fd.outliers
#' @export depth.BD
#' @export depth.ABD
#' @export depth.fd1
#' @export depth.fd2
#' @export depth.HR
#' @export depth.hM
#' @export depth.hM2
#' @export depth.RP1
#' @export depth.RP2
#' @export derivatives.est
#' @export DpairsPlot
#' @export L2metric
#' @export Cmetric
#' @export depth.sample
#' @export infimalRank

#' @title World population dataset
#' @name population
#' @docType data
#'
#' @description Historical world population data by countries.
#'
#' @usage data(population)
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords data
#'
#' @format A data frame with 233 rows and 67 variables
#' @source 
#' United Nations, Department of Economic and Social Affairs,
#' Population Division,
#' \url{https://esa.un.org/unpd/wpp/Download/Standard/Population/}, 
#' file \code{Total population - Both sexes}
#'
#' @details
#' World population data by a country, area or region as of 1 July 
#' of the year indicated. Figures are presented in thousands.
#'
#' @seealso \code{\link{population2010}}
#'
#' @examples
#' data(population)
#' head(population)
#' population.matrix = as.matrix(population[,1:66])
#' depth.fd1(population.matrix,population.matrix)

NULL

#' @title World population dataset, 2010 revision
#' @name population2010
#' @docType data
#'
#' @description Historical world population data by countries, revision 2010, for newer version of the data see \code{\link{population}}.
#' This dataset is included for compatibility with the analysis performed in the paper Nagy et al. (2016).
#'
#' @usage data(population2010)
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords data
#'
#' @format A data frame with 233 rows and 63 variables
#' @source 
#' United Nations, Department of Economic and Social Affairs,
#' Population Division,
#' \url{https://esa.un.org/unpd/wpp/Download/Standard/Population/}, 
#' file \code{Total population - Both sexes}
#'
#' @details
#' World population data by a country, area or region as of 1 July 
#' of the year indicated. Figures are presented in thousands.
#' 
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#'
#' @seealso \code{\link{population}}
#'
#' @examples
#' data(population2010)
#' head(population2010)
#' population.matrix = as.matrix(population2010[,1:61])
#' depth.fd1(population.matrix,population.matrix)

NULL

#' @title Fast kernel smoothing
#'
#' @description
#' Produces a kernel smoothed version of a function based on
#' the vectors given in the input. Bandwidth is selected using cross-validation.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords smoothing, kernel, functional data
#'
#' @details
#' A vector of the same length as \code{Tout}
#' corresponding to the values of the
#' function produced using kernel smoothing, is provided. Bandwidth is selected using the
#' \code{K}-fold cross-validation of randomly shuffled input values.
#'
#' @param T A vector of values in the interval [0,1].
#'
#' @param X A vector of the observed values of the contaminated function, 
#' corresponding to \code{T}. Must have the same length as \code{T}.
#'
#' @param Tout A vector of values in the interval [0,1] at which the
#' resulting smoothed function is evaluated.
#' 
#' @param kernel Kernel used for smoothing. Admissible values are \code{uniform}, 
#' \code{triangular}, \code{Epanechnikov}, \code{biweight}, \code{triweight} and \code{Gaussian}.
#' By default, \code{uniform} is used.
#' 
#' @param m Number of points in the grid for choosing the cross-validated bandwidth.
#' 
#' @param K Performs \code{K}-fold cross-validation based on randomly shuffled data.
#'
#' @return A vector corresponding to \code{Tout} of smoothed functional values.
#' 
#' @examples
#' d = 10
#' T = runif(d)
#' X = T^2+ rnorm(d,sd=.1)
#' Tout = seq(0,1,length=101)
#' 
#' plot(T,X)
#' lines(Tout,FKS(T,X,Tout))
#' lines(Tout,FKS(T,X,Tout,kernel="Epan"),col=2)
#' lines(Tout,FKS(T,X,Tout,kernel="Gaussian"),col=3)
#' lines(Tout,FKS(T,X,Tout,kernel="triweight"),col=4)
#' lines(Tout,FKS(T,X,Tout,kernel="biweight"),col=5)
#' lines(Tout,FKS(T,X,Tout,kernel="uniform"),col=2,lwd=2)

FKS = function(T,X,Tout,kernel=c("uniform","triangular","Epanechnikov","biweight","triweight","Gaussian"),m=51,K=20){
	# kernel smoothing
	# WORKS ONLY IN [0,1], functions on other domains must be transformed
	# produces directly the kernel smoother with CV value of h
	# Hmax 	the maximal H for CV
	# m		no of elements of H for CV
	# K		K-fold CV for bandwidth selection
	krn = c("uniform","triangular","Epanechnikov","biweight","triweight","Gaussian")
      kernel = match.arg(kernel)
	kernI = which(kernel==krn)
      if(sum(is.na(T))>0) stop("NA values in the T vector")
	ST = sort(T)
	eps = 10^(-6)
	Hmin = max(c(ST[1],(1-ST[length(ST)]),max(ST[-1] - ST[-length(ST)])/2))+eps
	H = seq(Hmin,1,length=m)
	#
	n = length(T)
	nR = max(1,floor(n/K))
	if(nR==1) K = n
	TRE = rep(NA,nR*K)
	XRE = rep(NA,nR*K)
	TNRE = rep(NA,K*(n-nR))
	XNRE = rep(NA,K*(n-nR))
	SH = sample(n,n)			# random shuffle of the points for CV
	for(i in 1:K){
		S = SH[(i-1)*nR+(1:nR)] 
		TRE[(i-1)*nR+(1:nR)] = T[S]
		XRE[(i-1)*nR+(1:nR)] = X[S]
		TNRE[(i-1)*(n-nR)+(1:(n-nR))] = T[-S]
		XNRE[(i-1)*(n-nR)+(1:(n-nR))] = X[-S]
		}
	Res = .Fortran("CVKERNSM",
		as.double(T),
		as.double(X),
		as.double(Tout),
		as.integer(length(T)),
		as.integer(length(Tout)),
		as.double(H),
		as.integer(length(H)),
            as.integer(kernI),
		as.double(TRE),
		as.double(XRE),
		as.double(TNRE),
		as.double(XNRE),
		as.integer(nR),		
		as.integer(K),
		as.double(rep(0,length(Tout)))
		)
	Res[Res[[15]]>10^6] = Inf
 	return(Res[[15]])
	}
      
#' @title Fast depth computation for univariate and bivariate random samples
#'
#' @description
#' Faster implementation of the halfspace and the simplicial depth. Computes the depth 
#' of a whole random sample of a univariate or a bivariate data in one run.	
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords halfspace depth, Tukey depth, simplicial depth, Liu depth
#'
#' @details
#' The function returns vectors of sample halfspace and simplicial depth values.
#'
#' @param A Univariate or bivariate points whose depth is computed, represented by a matrix of 
#' size \code{m*2}. \code{m} stands for the number of points, \code{d} is 1 for univariate and 2 
#' for bivariate data.
#'
#' @param B Random sample points with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of size \code{n*2}, where \code{n} is the sample size.
#'
#' @return Vector of length \code{m} of depth halfspace depth values is returned.
#' 
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#'
#' @examples
#' A = matrix(rnorm(2*100),ncol=2)
#' B = matrix(rnorm(2*150),ncol=2)
#' depth.sample(A,B)
#'
#' A = rnorm(100)
#' B = rnorm(150)
#' depth.sample(A,B)
#'
#' @seealso \code{\link{depth}}

depth.sample = function(A,B){
	# bivariate halfspace depth
	# A points whose depth I compute, M*2 matrix
	# B points wrt whose the depth is computed, N*2 matrix
      if(is.null(dim(A))){ # for univariate data
	A1 = as.vector(A)
      B1 = as.vector(B)	
	m = length(A)
	n = length(B)
	FD = .Fortran("dpth1",	
                  as.numeric(A1),		#	A1
                  as.numeric(B1),		#	B1
            	as.integer(m),		#	m
            	as.integer(n),		#	n
            	sdep=as.numeric(rep(-1,m)),
                  hdep=as.numeric(rep(-1,m))
      		)
      return(list(Simpl = FD$sdep, Half = FD$hdep))
	} else {
	A1 = as.vector(A[,1])
      A2 = as.vector(A[,2])
      B1 = as.vector(B[,1])
      B2 = as.vector(B[,2])
	m = dim(A)[1]
	n = dim(B)[1]
      if((dim(B)[2]!=2)|(dim(A)[2]!=2)) stop("Computation for two dimensions only")
	     FD = .Fortran("dpth2",	
                  as.numeric(A1),		#	A1
                  as.numeric(A2),		#	A2
                  as.numeric(B1),		#	B1
                  as.numeric(B2),		#	B2
            	as.integer(m),		#	m
            	as.integer(n),		#	n
            	sdep=as.numeric(rep(-1,m)),
                  hdep=as.numeric(rep(-1,m))
      		)
      return(list(Simpl = FD$sdep, Half = FD$hdep))
	}
	}

#' @title Univariate integrated and infimal depth for functional data
#'
#' @description
#' Usual, and order extended integrated and infimal depths for real-valued functional data based on the
#' halfspace and simplicial depth.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords integrated depth, infimal depth, functional data
#'
#' @details
#' The function returns vectors of sample integrated and infimal depth values.
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param order The order of the order extended integrated and infimal depths.
#' By default, this is set to \code{1}, meaning that the usual univariate depths of 
#' the functional values are computed. For \code{order=2} or \code{3}, the second
#' and the third order extended integrated and infimal depths are computed, 
#' respectively. 
#'
#' @param approx Number of approximations used in the computation of the order extended depth
#' for \code{order} greater than \code{1}. For \code{order=2}, the default
#' value is set to \code{0}, meaning that the depth is computed at all possible \code{d^order}
#' combinations of the points in the domain. For \code{order=3}, 
#' the default value is set to \code{101}. When \code{approx} is a positive integer, \code{approx}
#' points are randomly sampled in \code{[0,1]^order} and at these points the \code{order}-variate depths of the
#' corresponding functional values are computed.
#'
#' @return Four vectors of length \code{m} of depth values are returned:
#'	\itemize{
#'	\item \code{Simpl_FD} the integrated depth based on the simplicial depth,
#'	\item \code{Half_FD} the integrated depth based on the halfspace depth,
#'	\item \code{Simpl_ID} the infimal depth based on the simplicial depth,
#'	\item \code{Half_ID} the infimal depth based on the halfspace depth.
#'	}
#' In addition, two vectors of length \code{m} of the relative area of smallest depth values is returned:
#'	\itemize{
#'	\item \code{Simpl_IA} the proportions of points at which the depth \code{Simpl_ID} was attained,
#'	\item \code{Half_IA} the proportions of points at which the depth \code{Half_ID} was attained.
#'	}
#' The values \code{Simpl_IA} and \code{Half_IA} are always in the interval [0,1]. 
#' They introduce ranking also among functions having the same
#' infimal depth value - if two functions have the same infimal depth, the one with larger infimal area
#' \code{IA} is said to be less central. 	
#' For \code{order=2} and \code{m=1}, two additional matrices of pointwise depths are also returned:
#'	\itemize{
#'    \item \code{PSD} the matrix of size \code{d*d} containing the computed 
#'    pointwise bivariate simplicial depths used for the computation of \code{Simpl_FD} and \code{Simpl_ID},
#'    \item \code{PHD} the matrix of size \code{d*d} containing the computed 
#'    pointwise bivariate halfspace depths used for the computation of \code{Half_FD} and \code{Half_ID}.
#'	}
#' For \code{order=3}, only \code{Half_FD} and \code{Half_ID} are provided.
#' 
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#'
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.fd1(A,B)
#' depth.fd1(A,B,order=2)
#' depth.fd1(A,B,order=3,approx=51)
#'
#' @seealso \code{\link{depth.fd2}}, \code{\link{infimalRank}}

depth.fd1 = function(A,B,order=1,approx=0){
	# univariate integrated depth
	# A functions whose depth I compute, M*D matrix
	# B functions wrt whose the depth is computed, N*D matrix
	# both 1dimensional, n*d, n nr of functions, d dimensionality
	if(order==1){
           A1 = as.vector(A)
	     B1 = as.vector(B)
	     d = dim(A)[2]
	     m = dim(A)[1]
	     n = dim(B)[1]
	     if(dim(B)[2]!=d) stop("dimension mismatch")
	     FD = .Fortran("funD1",	
            	as.numeric(A1),		#	A
             	as.numeric(B1),		#	B
            	as.integer(m),		#	m
            	as.integer(n),		#	n
                 	as.integer(d),		#	d
            	funsdep=as.numeric(rep(-1,m)),	
            	funhdep=as.numeric(rep(-1,m)),
      		fIsdep =as.numeric(rep(-1,m)),
      		fIhdep =as.numeric(rep(-1,m)),
			IAsdep =as.integer(rep(-1,m)),
			IAhdep =as.integer(rep(-1,m))
      		)
      	return(list(	Simpl_FD = FD$funsdep, Half_FD = FD$funhdep,
      				Simpl_ID = FD$fIsdep, Half_ID = FD$fIhdep, 
					Simpl_IA = FD$IAsdep/d, Half_IA = FD$IAhdep/d ))
            }
      if(order==2) return(DiffDepth(A,B,approx))
      if(order==3) return(DiffDepth3D(A,B,approx))
	}

#' @title Fast computation of the \eqn{L^2} metric for sets of functional data
#'
#' @description
#' Returns the matrix of \eqn{L^2} distances between two sets of functional data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords \eqn{L^2} metric, functional data
#'
#' @details
#' For two sets of functional data of sizes \code{m} and \code{n}
#' represented by matrices of their functional values, 
#' this function returns the symmetric matrix of size \code{m*n} whose entry in the
#' \code{i}-th row and \code{j}-th column is the approximated \eqn{L^2} distance of the 
#' \code{i}-th function from the first set, and the \code{j}-th function from the second set.
#' This function is utilized in the computation of the h-mode depth.
#'
#' @param A Functions of the first set, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param A Functions of the second set, represented by a matrix of its functional values of 
#' size \code{n*d}. \code{n} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @return A symmetric matrix of the distances of the functions of size \code{m*n}.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' L2metric(A,B)
#'
#' @seealso \code{\link{depth.hM}}

L2metric = function(A,B){
	# computes fast approximation of L2 distance between fctions A and B
	M = .Fortran("metrl2",
		as.numeric(A),
		as.numeric(B),
		as.integer(m<-nrow(A)),
		as.integer(n<-nrow(B)),
		as.integer(d<-length(as.numeric(A))/nrow(A)),
		m = as.numeric(rep(-1,m*n)))$m
	return(M = matrix(M,nrow=m))
	}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.hM}}

depth.M = function(A,B,q=.2){
	# h-mode depth for the L2 metric
	mdist = L2metric(B,B)
	mdist2 = L2metric(A,B)
	hq2 = quantile(mdist[mdist>0],probs=q,type=1)	# probs=.2 as in Cuevas et al. (2007)
	return(rowSums(dnorm(mdist2/hq2)))
	}

#' @title Fast computation of the uniform metric for sets of functional data
#'
#' @description
#' Returns the matrix of \eqn{C} (uniform) distances between two sets of functional data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords uniform metric, functional data
#'
#' @details
#' For two sets of functional data of sizes \code{m} and \code{n}
#' represented by matrices of their functional values, 
#' this function returns the symmetric matrix of size \code{m*n} whose entry in the
#' \code{i}-th row and \code{j}-th column is the approximated \eqn{C} (uniform) distance of the 
#' \code{i}-th function from the first set, and the \code{j}-th function from the second set.
#' This function is utilized in the computation of the h-mode depth.
#'
#' @param A Functions of the first set, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param A Functions of the second set, represented by a matrix of its functional values of 
#' size \code{n*d}. \code{n} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @return A symmetric matrix of the distances of the functions of size \code{m*n}.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' Cmetric(A,B)
#'
#' @seealso \code{\link{depth.hM}}

Cmetric = function(A,B){
	# computes fast approximation of \eqn{C} distance between fctions A and B
	M = .Fortran("metrC",
		as.numeric(A),
		as.numeric(B),
		as.integer(m<-nrow(A)),
		as.integer(n<-nrow(B)),
		as.integer(d<-length(as.numeric(A))/nrow(A)),
		m = as.numeric(rep(-1,m*n)))$m
	return(M = matrix(M,nrow=m))
	}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.hM}}

depth.MC = function(A,B,q=.2){
	# h-mode depth for the \eqn{C} metric
	mdist = Cmetric(B,B)
	mdist2 = Cmetric(A,B)
	hq2 = quantile(mdist[mdist>0],probs=q,type=1)	# probs=.2 as in Cuevas et al. (2007)
	return(rowSums(dnorm(mdist2/hq2)))
	}
      
#' @title h-mode depth for functional data
#'
#' @description
#' The h-mode depth of functional real-valued data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords h-mode depth, functional data
#'
#' @details
#' The function returns the vectors of the sample h-mode depth values. The kernel 
#' used in the evaluation is the standard Gaussian kernel, the bandwidth value is chosen
#' as a quantile of the non-zero distances between the random sample curves.
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param norm The norm used for the computation of the depth. Two possible 
#' choices are implemented: \code{C} for the uniform norm of continuous functions, 
#' and \code{L2} for the \eqn{L^2} norm of integrable functions.
#'
#' @param q The quantile used to determine the value of the bandwidth \eqn{h}
#' in the computation of the h-mode depth. \eqn{h} is taken as the \code{q}-quantile of
#' all non-zero distances between the functions \code{B}. By default, this value is set
#' to \code{q=0.2}, in accordance with the choice of Cuevas et al. (2007).
#'
#' @return A vector of length \code{m} of the h-mode depth values.
#' 
#' @references Cuevas, A., Febrero, M. and Fraiman, R.  (2007), Robust estimation and classification for functional data via projection-based depth notions, 
#' \emph{Computational Statistics} \bold{22} (3), 481  496.
#'
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.hM(A,B)
#' depth.hM(A,B,norm="L2")
      
depth.hM = function(A, B, norm = c("C","L2"), q=.2){
      norm = match.arg(norm)
      switch(norm,
            C = depth.MC(A,B,q),
            L2 = depth.M(A,B,q)
            )
      }

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDKernL = function(b,v,J=3){
	# v is a matrix m x eval
	m = dim(v)[1]                                   # sample size
	eval = length(b)                                
	com = combn(m,J)
	poc = dim(com)[2]
	s = .Fortran("AdjLP",
		as.integer(eval),
		as.integer(J),
		as.integer(m),
		as.integer(poc),
		as.integer(as.vector(com)),
		as.numeric(b),
		as.numeric(v),
		dj = as.numeric(1))$dj
	return(s)
}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDL = function(b,v,J=3,K=1){
	eval = length(b)
	m = dim(v)[1]                                   # sample size
	sam = sample.int(m,m)                           # shuffle
	if (K==0) K=1                                   # make sure K!=0
	nk = m%/%K                                      # subsample size
	nlast = m %% K+nk                               # last subsample size
	Dpom = rep(NA,K)	   					# subsample depth
	if (K>1){
	        for (k in 1:(K-1)) 
	                if (eval>1) Dpom[k] = AdjBDKernL(b,v[sam[((k-1)*nk+1):(k*nk)],],J)
	                        else  Dpom[k] = AdjBDKernL(b,t(as.matrix(v[sam[((k-1)*nk+1):(k*nk)],])),J)
	        }
	{if (nlast>0){	
	        if (eval>1) Dpom[K]= AdjBDKernL(b,v[sam[((K-1)*nk+1):m],],J)
	                else  Dpom[K]= AdjBDKernL(b,t(as.matrix(v[sam[((K-1)*nk+1):m],])),J)
     	   }
        else (K=K-1)}
	return(mean(Dpom[1:K]))
}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDsampleL = function(A,B,J=3,K=1){
	m = dim(A)[1]
	hlb = rep(NA,m)
	for (i in 1:m) hlb[i] = AdjBDL(A[i,],B,J,K)
	return(hlb)
}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDKernC = function(b,v,J=3){
	# v ma rozmery m x eval
	m = dim(v)[1]						# sample size
	eval = length(b)					
	com = combn(m,J)	
	poc = dim(com)[2]
	s = .Fortran("AdjC",
		as.integer(eval),
		as.integer(J),
		as.integer(m),
		as.integer(poc),
		as.integer(as.vector(com)),
		as.numeric(b),
		as.numeric(v),
		dj = as.numeric(1))$dj
	return(s)
}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDC = function(b,v,J=3,K=1){
	# v is a matrix m x eval
	eval = length(b)
	m = dim(v)[1]                                   # sample size
	sam = sample.int(m,m)                           # shuffle
	if (K==0) K=1                                   # make sure K!=0
	nk = m%/%K                                      # subsample size
	nlast = m %% K+nk                               # last subsample size
	Dpom = rep(NA,K)	   					# subsample depth
	if (K>1){
		for (k in 1:(K-1)) 
			if (eval>1) Dpom[k] = AdjBDKernC(b,v[sam[((k-1)*nk+1):(k*nk)],],J)
				else  Dpom[k] = AdjBDKernC(b,t(as.matrix(v[sam[((k-1)*nk+1):(k*nk)],])),J)
		}
	{if (nlast>0){
		if (eval>1) Dpom[K]= AdjBDKernC(b,v[sam[((K-1)*nk+1):m],],J)
			else  Dpom[K]= AdjBDKernC(b,t(as.matrix(v[sam[((K-1)*nk+1):m],])),J)
		}
		else (K=K-1)}
	return(mean(Dpom[1:K]))
}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.ABD}}

AdjBDsampleC = function(A,B,J=3,K=1){
	m = dim(A)[1]
	hlb = rep(NA,m)
	for (i in 1:m) hlb[i] = AdjBDC(A[i,],B,J,K)
	return(hlb)
}

#' @title Adjusted band depth for functional data
#'
#' @description
#' The adjusted band depth 
#' of functional real-valued data based on either the
#' \eqn{C} (uniform) norm, or on the \eqn{L^2} norm of functions.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords band depth, functional data
#'
#' @details
#' The function returns the vector of the sample adjusted band depth values. The kernel 
#' used in the evaluation is the function \eqn{K(u) = exp(-u)}.
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param norm The norm used for the computation of the depth. Two possible 
#' choices are implemented: \code{C} for the uniform norm of continuous functions, 
#' and \code{L2} for the \eqn{L^2} norm of integrable functions. 
#'
#' @param J The order of the adjusted band depth, that is the maximal number of functions
#' taken in a band. Acceptable values are \code{2}, \code{3},... By default this value is set to \code{2}. 
#' Note that this is NOT the order as
#' defined in the order-extended version of adjusted band depths in Nagy et al. (2016), used
#' for the detection of shape outlying curves.
#' 
#' @param K Number of sub-samples of the functions from \code{B} taken to speed up the
#' computation. By default, sub-sampling is not performed. Values of \code{K} larger than \code{1}
#' result in an approximation of the adjusted band depth.
#'
#' @return A vectors of length \code{m} of the adjusted band depths.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.ABD(A,B)
#' depth.ABD(A,B,norm="L2")
#'
#' @seealso \code{\link{depth.BD}}
#'
#' @references Gijbels, I., Nagy, S. (2015), Consistency of non-integrated depths for functional data,
#' \emph{J. Multivariate Anal.} \bold{140}, 259 - 282.
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.

depth.ABD = function(A, B, norm = c("C","L2"), J=2, K=1){
      norm = match.arg(norm)
      switch(norm,
            C = AdjBDsampleC(A,B,J,K),
            L2 = AdjBDsampleL(A,B,J,K)
            )
      }

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.fd1}}

DiffDepth = function(A,B,approx=0){
	# A functions whose depth I compute
	# B functions wrt whose the depth is computed
	# both 1dimensional, n*d, n nr of functions, d dimensionality
	# approx is number of approximations to be used, 0 for full computation

	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	if(dim(B)[2]!=d) stop("dimension mismatch")
	Av = as.vector(A)
	Bv = as.vector(B)
	if (approx>0){
		C = combn(1:d,2)		# all couples 
		if (approx>=ncol(C)){ 
			ind.sample = 1:ncol(C) 
			approx = ncol(C) } else 
			ind.sample = sample.int(ncol(C),approx)
		RN = matrix(C[,ind.sample],nrow=2)		#	RN = random numbers of size 2*REP from 1:d
		} else RN = 0		
	FD = .Fortran("DiffD",	
      	as.numeric(Av),		#	A
       	as.numeric(Bv),		#	B
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
		as.integer(approx),	#	REP
		as.integer(RN),		#	RN
      	funsdep=as.numeric(rep(-1,m)),	
      	funhdep=as.numeric(rep(-1,m)),
      	funsdepm=as.numeric(rep(-1,m)),	
      	funhdepm=as.numeric(rep(-1,m)),
		Psdep = as.numeric(rep(-1,d*d)),
		Phdep = as.numeric(rep(-1,d*d)),
		IAsdep =as.integer(rep(-1,m)),
		IAhdep =as.integer(rep(-1,m)))
	if(approx==0){
		S_IA = FD$IAsdep/(d^2)
		H_IA = FD$IAhdep/(d^2)
		} else {
		S_IA = FD$IAsdep/(approx)
		H_IA = FD$IAhdep/(approx)		
		}
	if (m>1) return(list(	Simpl_FD = FD$funsdep, Half_FD = FD$funhdep,
					Simpl_ID = FD$funsdepm, Half_ID = FD$funhdepm,
					Simpl_IA = S_IA, Half_IA = H_IA))
	if ((m==1)&(approx==0)){
		PSD = matrix(FD$Psdep,ncol=d)
		PSD = pmax(PSD,0)
		PSD = PSD + t(PSD)
		diag(PSD) = NA
		PHD = matrix(FD$Phdep,ncol=d)
		PHD = pmax(PHD,0)
		PHD = PHD + t(PHD)
		diag(PHD) = NA
		return(list(	Simpl_FD = FD$funsdep, Half_FD = FD$funhdep,
					Simpl_ID = FD$funsdepm, Half_ID = FD$funhdepm,
					PSD = PSD, PHD = PHD))
		}
	if ((m==1)&(approx>0)){
		PSD = matrix(FD$Psdep,ncol=d)
		for(i in 1:approx) PSD[RN[2*i-1],RN[2*i]]=PSD[RN[2*i],RN[2*i-1]]	# symmetrize
		PSD[PSD==-1] = NA
		PHD = matrix(FD$Phdep,ncol=d)
		for(i in 1:approx) PHD[RN[2*i-1],RN[2*i]]=PHD[RN[2*i],RN[2*i-1]]	# symmetrize
		PHD[PHD==-1] = NA
		return(list(	Simpl_FD = FD$funsdep, Half_FD = FD$funhdep, 
					Simpl_ID = FD$funsdepm, Half_ID = FD$funhdepm,
					PSD = PSD, PHD = PHD))
		}
	}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{depth.fd1}}
      
DiffDepth3D = function(A,B,approx=101){
	
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	if(dim(B)[2]!=d) stop("dimension mismatch")
	D = rep(0,m)		# integrated depth
	DI = rep(2,m)		# infimal depth
	IA = rep(0,m)		# infimal area
	if(approx==0) approx=101
	for(a in 1:approx){
		I = sample.int(d,3)
		for(N in 1:m){
			DT = depth(c(A[N,I[1]],A[N,I[2]],A[N,I[3]]),cbind(B[,I[1]],B[,I[2]],B[,I[3]]),approx=FALSE)
			D[N] = D[N] + DT
			if(DT==DI[N]) IA[N] = IA[N]+1
			if(DT<DI[N]) IA[N] = 1
			DI[N] = min(DI[N],DT)
			}		
		}
	D = D/approx
	IA = IA/approx
	return(list(Half_FD=D,Half_ID=DI,Half_IA=IA))
	}

#' @title Bivariate h-mode depth for functional data based on the \eqn{L^2} metric
#'
#' @description
#' The h-mode depth 
#' of functional bivariate data (that is, data of the form \eqn{X:[a,b] \to R^2},
#' or \eqn{X:[a,b] \to R} and the derivative of \eqn{X}) based on the
#' \eqn{L^2} metric of functions.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords h-mode depth, functional data, derivatives
#'
#' @details
#' The function returns the vectors of sample h-mode depth values. The kernel 
#' used in the evaluation is the standard Gaussian kernel, the bandwidth value is chosen
#' as a quantile of the non-zero distances between the random sample curves.
#'
#' @param A Functions whose depth is computed, represented by an array of 
#' size \code{m*d*2}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated. In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by an array of 
#' size \code{n*d*2}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated.  In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#' The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param q The quantile used to determine the value of the bandwidth \eqn{h}
#' in the computation of the h-mode depth. \eqn{h} is taken as the \code{q}-quantile of
#' all non-zero distances between the functions \code{B}. By default, this value is set
#' to \code{q=0.2}, in accordance with the choice of Cuevas et al. (2007).
#'
#' @return Three vectors of length \code{m} of h-mode depth values are returned:
#'	\itemize{
#'	\item \code{hM} the unscaled h-mode depth,
#'	\item \code{hM_norm} the h-mode depth \code{hM} linearly transformed so that its range is [0,1],
#'	\item \code{hM_norm2} the h-mode depth \code{FD} linearly transformed by a transformation such that 
#'    the range of the h-mode depth of \code{B} with respect to \code{B} is [0,1]. This depth may give negative values.
#'	}
#'
#' @references Cuevas, A., Febrero, M. and Fraiman, R.  (2007), Robust estimation and classification for functional data via projection-based depth notions, 
#' \emph{Computational Statistics} \bold{22} (3), 481  496.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#'
#' A2 = derivatives.est(A,deriv=c(0,1))
#' B2 = derivatives.est(B,deriv=c(0,1))
#' 
#' depth.hM2(A2,B2)
#'
#' depth.hM2(A2,B2)$hM
#' depth.hM2(cbind(A2[,,1],A2[,,2]),cbind(B2[,,1],B2[,,2]))$hM
#' # the two expressions above should give the same result
#'
#' @seealso \code{\link{depth.hM}}

depth.hM2 = function(A,B,q=.2){
# h-Mode depth Cuevas_etal2007
# A functions whose depth is computed :	either M*D matrix (M functions, D points per function), 
#							or M*D*2 array (2 derivative levels), 
# B functions of random sample :		as A
# q :							quantile, bandwidth value in the resulting kernel estimate
#							computes the same as depth.M (with cbind-ed 2 levels if derivatives are involved)
#							depth.M(cbind(A[,,1],A[,,2]),cbind(B[,,1],B[,,2]))
#							depth.M2(A,B)$FD
#							depth.M2(cbind(A[,,1],A[,,2]),cbind(B[,,1],B[,,2]))$FD
# 							all should give the same result

# q=.2 Cuevas et al, 0.15 default v usc.fda
	M = .Fortran(	"funMD",
		as.numeric(A),
		as.numeric(B),
		as.integer(m<-nrow(A)),
		as.integer(n<-nrow(B)),
		as.integer(d<-length(as.numeric(A))/nrow(A)),
		as.numeric(q),
		md = as.numeric(rep(-1,m))
		)$md
#	because of scaling in depth.mode() in fda.usc
	M.ori = .Fortran(	"funMD",
		as.numeric(B),
		as.numeric(B),
		as.integer(n),
		as.integer(n),
		as.integer(d),
		as.numeric(q),
		md = as.numeric(rep(-1,n))
		)$md
	Mn = (M-min(M))/(max(M)-min(M))
	Mn2 = (M-min(M.ori))/(max(M.ori)-min(M.ori))
	return(list(hM = M, hM_norm = Mn, hM_norm2 = Mn2))
	}

#' @title Univariate random projection depths for functional data
#'
#' @description
#' Random projection depth and random functional depth for functional data.	
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords random projection depth, random functional depth, random Tukey depth, functional data
#'
#' @details
#' The function returns the vectors of sample random projection, and random functional depth values. 
#' The random projection depth described in Cuevas et al. (2007) is based on the average univariate depth
#' of one-dimensional projections of functional data. The projections are taken randomly as a sample of standard
#' normal \code{d}-dimensional random variables, where \code{d} stands for the dimensionality of the discretized
#' functional data. 
#'
#' The random functional depth (also called random Tukey depth, or random halfspace depth) is described in
#' Cuesta-Albertos and Nieto-Reyes (2008). The functional data are projected into the real line in random 
#' directions as for the random projection depths. Afterwards, an approximation of the halfspace (Tukey) depth
#' based on this limited number of univariate projections is assessed. 
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param nproj Number of projections taken in the computation of the random projection depth. By default taken
#' to be \code{51}.
#'
#' @param nproj2 Number of projections taken in the computation of the random functional depth. By default taken
#' to be \code{5}. \code{nproj2} should be much smaller than \code{d}, the dimensionality of the discretized 
#' functional data.
#'
#' @return Three vectors of depth values of length \code{m} are returned:
#'	\itemize{
#'	\item \code{Simpl_FD} the random projection depth based on the univariate simplicial depth,
#'	\item \code{Half_FD} the random projection depth based on the univariate halfspace depth,
#'	\item \code{RHalf_FD} the random halfspace depth.
#'	}
#'
#' @references Cuevas, A., Febrero, M. and Fraiman, R.  (2007), Robust estimation and classification for functional data via projection-based depth notions, 
#' \emph{Computational Statistics} \bold{22} (3), 481  496.
#'
#' @references Cuesta-Albertos, J.A. and Nieto-Reyes, A. (2008), The random Tukey depth, 
#' \emph{Computational Statistics & Data Analysis} \bold{52} (11), 4979 - 4988.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.RP1(A,B)
#'
#' @seealso \code{\link{depth.RP2}}
      
depth.RP1 = function(A,B,nproj=50,nproj2=5){
# simple 1D projection depth
# nproj nr of projections taken
#
#	SFD : projection \eqn{L^2} -> R -> mean of univariate Simplicial Depth (nproj projections)
#	HFD : projection \eqn{L^2} -> R -> mean of univariate Halfspace Depth (nproj projections)
#	RFD : projection \eqn{L^2} -> R -> infimum of univariate Halfspace Depths (nproj2 projections)

	A1 = as.vector(A)
	B1 = as.vector(B)
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	V = rnorm(nproj*d)
	if(dim(B)[2]!=d) stop("dimension mismatch")
	FD = .Fortran("funRPD1",	
       	as.numeric(A1),		#	A
      	as.numeric(B1),		#	B
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
		as.integer(nproj),	#	nproj
		as.integer(nproj2),	#	nproj2
		as.numeric(V),		#	projections
      	funsdep=as.numeric(rep(-1,m)),	
      	funhdep=as.numeric(rep(-1,m)),
		funrdep=as.numeric(rep(-1,m)))
	return(list(Simpl_FD = FD$funsdep, Half_FD = FD$funhdep, RHalf_FD = FD$funrdep))
	}

#' @title Bivariate integrated and infimal depth for functional data
#'
#' @description
#' Integrated and infimal depths 
#' of functional bivariate data (that is, data of the form \eqn{X:[a,b] \to R^2},
#' or \eqn{X:[a,b] \to R} and the derivative of \eqn{X}) based on the
#' bivariate halfspace and simplicial depths.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords integrated depth, infimal depth, functional data, derivatives
#'
#' @details
#' The function returns the vectors of sample integrated and infimal depth values.
#'
#' @param A Functions whose depth is computed, represented by an array of 
#' size \code{m*d*2}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated. In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by an array of 
#' size \code{n*d*2}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated.  In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#' The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @return Four vectors of length \code{m} are returned:
#'	\itemize{
#'	\item \code{Simpl_FD} the integrated depth based on the bivariate simplicial depth,
#'	\item \code{Half_FD} the integrated depth based on the bivariate halfspace depth,
#'	\item \code{Simpl_ID} the infimal depth based on the bivariate simplicial depth,
#'	\item \code{Half_ID} the infimal depth based on the bivariate halfspace depth.
#'	}
#' In addition, two vectors of length \code{m} of the relative area of smallest depth values is returned:
#'	\itemize{
#'	\item \code{Simpl_IA} the proportions of points at which the depth \code{Simpl_ID} was attained,
#'	\item \code{Half_IA} the proportions of points at which the depth \code{Half_ID} was attained.
#'	}
#' The values \code{Simpl_IA} and \code{Half_IA} are always in the interval [0,1]. 
#' They introduce ranking also among functions having the same
#' infimal depth value - if two functions have the same infimal depth, the one with larger infimal area
#' \code{IA} is said to be less central. 	
#'
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#'
#' A2 = derivatives.est(A,deriv=c(0,1))
#' B2 = derivatives.est(B,deriv=c(0,1))
#' depth.fd2(A2,B2)
#'
#' @seealso \code{\link{depth.fd1}}, \code{\link{infimalRank}}
      
depth.fd2 = function(A,B){
	# A functions whose depth I compute
	# B functions wrt whose the depth is computed
	# both 2dimensional, n*d*2, n nr of functions, d dimensionality
	# now provides also infimal depth (inf_D2)

	A1 = as.vector(A[,,1])
	A2 = as.vector(A[,,2])
	B1 = as.vector(B[,,1])
	B2 = as.vector(B[,,2])
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	if(dim(B)[2]!=d) stop("dimension mismatch")
	FD = .Fortran("funD2",	
      	as.numeric(A1),		#	A1
      	as.numeric(A2),		#	A2
      	as.numeric(B1),		#	B1
      	as.numeric(B2),		#	B2
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
      	funsdep=as.numeric(rep(-1,m)),	
      	funhdep=as.numeric(rep(-1,m)),
		fIsdep =as.numeric(rep(-1,m)),
		fIhdep =as.numeric(rep(-1,m)),
		IAsdep =as.integer(rep(-1,m)),
		IAhdep =as.integer(rep(-1,m))
		)
	return(list(	Simpl_FD = FD$funsdep, Half_FD = FD$funhdep,
				Simpl_ID = FD$fIsdep,	Half_ID = FD$fIhdep,
				Simpl_IA = FD$IAsdep/d, Half_IA = FD$IAhdep/d ))
	}

#' @title Bivariate random projection depths for functional data
#'
#' @description
#' Double random projection depths of functional bivariate data (that is, data of the form \eqn{X:[a,b] \to R^2},
#' or \eqn{X:[a,b] \to R} and the derivative of \eqn{X}).	
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords double random projection depth, functional data, derivatives
#'
#' @details
#' The function returns the vectors of sample double random projection depth values. 
#' The double random projection depths are described in Cuevas et al. (2007). They are of two types: RP2 type, and
#' RPD type. Both types of depths are based on bivariate projections of the bivariate functional data. 
#' These projections are taken randomly as a sample of standard
#' normal \code{d}-dimensional random variables, where \code{d} stands for the dimensionality of the discretized
#' functional data. For RP2 type depths, the average bivariate depth of the projected quantities is assessed.
#' For RPD type depths, further univariate projections of these bivariate projected quantities are evaluated, and
#' based on these final univariate quantities, the average univariate depth is computed.
#'
#' @param A Functions whose depth is computed, represented by an array of 
#' size \code{m*d*2}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated. In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by an array of 
#' size \code{n*d*2}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated.  In the third dimension of the array, 
#' the functional values, and the values of the derivatives of these functions, are provided, respectively.
#' The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param nproj Number of projections taken in the computation of the double random projection depth. By default taken
#' to be \code{51}.
#'
#' @return Five vectors of length \code{m} are returned:
#'	\itemize{
#'	\item \code{Simpl_FD} the double random projection depth RP2 based on the bivariate simplicial depth,
#'	\item \code{Half_FD} the double random projection depth RP2 based on the bivariate halfspace depth,
#'	\item \code{hM_FD} the double random projection depth RP2 based on the bivariate h-mode depth,
#'	\item \code{Simpl_DD} the double random projection depth RPD based on the univariate simplicial depth,
#'	\item \code{Half_DD} the random projection depth RPD based on the univariate halfspace depth,
#'	}
#'
#' @references Cuevas, A., Febrero, M. and Fraiman, R.  (2007), Robust estimation and classification for functional data via projection-based depth notions, 
#' \emph{Computational Statistics} \bold{22} (3), 481  496.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#'
#' A2 = derivatives.est(A,deriv=c(0,1))
#' B2 = derivatives.est(B,deriv=c(0,1))
#' depth.RP2(A2,B2)
#'
#'
#' @seealso \code{\link{depth.RP1}}
      
depth.RP2 = function(A,B,nproj=51){
# double projection depth
# nproj nr of projections taken
#
#	SFD :	(\eqn{L^2})^2 -> R^2 -> 2D Mean Simplicial Depth
#	HFD : (\eqn{L^2})^2 -> R^2 -> 2D Mean Halfspace Depth
#	MFD : (\eqn{L^2})^2 -> R^2 -> 2D Mean h-Mode Depth
#	SDD : (\eqn{L^2})^2 -> R^2 -> R^1 -> Mean Simplicial Depth
#	HDD : (\eqn{L^2})^2 -> R^2 -> R^1 -> Mean Halfspace Depth
#
	q = .2	# quantile for modal depth
	A1 = as.vector(A[,,1])
	A2 = as.vector(A[,,2])
	B1 = as.vector(B[,,1])
	B2 = as.vector(B[,,2])
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	V = rnorm(nproj*d+nproj*2)
	if(dim(B)[2]!=d) stop("dimension mismatch")
	FD = .Fortran("funRPD2",	
       	as.numeric(A1),		#	A1
      	as.numeric(A2),		#	A2
      	as.numeric(B1),		#	B1
      	as.numeric(B2),		#	B2
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
		as.integer(nproj),	#	nproj
		as.numeric(V),		#	projections
		as.numeric(q),		# 	q
      	funsdep=as.numeric(rep(-1,m)),	
      	funhdep=as.numeric(rep(-1,m)),
		funmdep=as.numeric(rep(-1,m)),
		funsddep=as.numeric(rep(-1,m)),
		funhddep=as.numeric(rep(-1,m)))
	return(list(Simpl_FD = FD$funsdep, Half_FD = FD$funhdep, hM_FD = FD$funmdep, Simpl_DD = FD$funsddep, Half_DD = FD$funhddep))
	}
      
#' @title Half-region depth for functional data
#'
#' @description
#' The half-region depth 
#' for functional real-valued data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords half-region depth, functional data
#'
#' @details
#' The function returns the vector of the sample half-region depth values.
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @return A vector of length \code{m} of the half-region depth values.
#'
#' @references Lpez-Pintado, S. and Romo, J. (2011), A half-region depth for functional data,
#' \emph{Computational Statistics & Data Analysis} \bold{55} (4), 1679  1695.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.HR(A,B)

depth.HR = function(A,B){
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	if(dim(B)[2]!=d) stop("dimension mismatch")
	FD = .Fortran("HRD",	
       	as.numeric(A),		#	A
      	as.numeric(B),		#	B
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
      	FD=as.numeric(rep(-1,m)))
	return(FD$FD)
	}

#' @title Band depth for functional data
#'
#' @description
#' The (unadjusted) band depth 
#' for functional real-valued data of order \code{J=2}.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords band depth, functional data
#'
#' @details
#' The function returns the vector of the sample (unadjusted) band depth values.
#'
#' @param A Functions whose depth is computed, represented by a matrix of its functional values of 
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @return A vector of length \code{m} of the band depth values.
#' 
#' @references Lpez-Pintado, S. and Romo, J. (2009), On the concept of depth for functional data,
#' \emph{J. Amer. Statist. Assoc.} \bold{104} (486), 718  734.
#'
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' depth.BD(A,B)
#'
#' @seealso \code{\link{depth.ABD}}, \code{\link{depth.fd1}}
       
depth.BD = function(A,B){
	d = dim(A)[2]
	m = dim(A)[1]
	n = dim(B)[1]
	if(dim(B)[2]!=d) stop("dimension mismatch")
	FD = .Fortran("BD",	
       	as.numeric(A),		#	A
      	as.numeric(B),		#	B
      	as.integer(m),		#	m
      	as.integer(n),		#	n
      	as.integer(d),		#	d
      	FD=as.numeric(rep(-1,m))
		)
	return(FD$FD)
	}

#' @title Adjusted ranking of functional data based on the infimal depth
#'
#' @description
#' Returns a vector of adjusted depth-based ranks for infimal depth for functional data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords rank, data depth, infimal depth, functional data
#'
#' @details
#' Infimal depths for functional data tend to give to many functional observations the same 
#' value of depth. Using this function, the data whose depth is the same is ranked according
#' to the infimal area indicator. This indicator is provided in functions \code{depth.fd1} along
#' the value of the infimal depth. 
#'
#' @param ID The vector of infimal depths of the curves of length \code{n}.
#'
#' @param IA The vector of the infimal areas corresponding to the infimal depths from \code{ID}
#' of length \code{n}.
#'
#' @param ties.method Parameter for breaking ties in infimal area index. By default \code{max}, see 
#' \code{rank}.
#' 
#' @return A vector of length \code{n}. Low depth values mean high ranks, i.e. potential outlyingess. 
#' If some of the infimal depths are identical, the ranking of these functions is made according to the 
#' values of the infimal area. There, higher infimal area index means higher rank, i.e. non-centrality. 
#' 
#' @examples
#' data(population)
#' X = as.matrix(population[1:20,1:66])
#' D = depth.fd1(X,X)
#' infimalRank(D$Half_ID,D$Half_IA) 
#' 
#' ID = c(0,1,0,0,0,1,1)
#' IA = c(2,3,1,0,2,4,1)
#' infimalRank(ID,IA)


infimalRank = function(ID,IA,ties.method="max"){
# finds the adjusted rank for appropriate for the infimal depth for functional data
# ID is the vector of infimal depths
# IA is the vector of the corresponding infimal areas
# returns a vector of ranks
	n = length(ID)
	if(length(IA)!=n) stop("Lengths of the vectors differ")
	U = sort(unique(ID),decreasing=TRUE)
	R = rep(NA,n)
	cR = 0						# currently assigned rank	
	for(u in U){
		Iu = (ID==u)
		R[Iu] = cR+rank(IA[Iu],ties.method=ties.method)
		cR = sum(!is.na(R))
		}
	return(R)
	}
      
#' @title Estimation of the first two derivatives for functional data
#'
#' @description
#' Returns an array of the values of derivatives of functional data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords derivatives, kernel, functional data
#'
#' @details
#' If the input \code{X} is a matrix of size \code{m*d}, 
#' the function returns an array of dimension \code{m*d*nd}, where 
#' in the third dimension of the array the estimated values of the derivatives of \code{X}
#' are given.
#' \code{nd} here stands for \code{1}, \code{2} or \code{3}, depending on how many derivatives of \code{X} are
#' requested computed. For the estimation, the function \code{D1ss} from the package
#' \code{sfsmisc} is utilized. 
#'
#' @param X Functions whose derivatives are computed, represented by a matrix of its functional values of
#' size \code{m*d}. \code{m} stands for the number of functions, \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{m} functions are evaluated.
#'
#' @param spar If provided, this parameter is passed to the function \code{D1ss} from package \code{sfsmisc}
#' as the value of the smoothing spline parameter in order to numerically approximate
#' the derivatives of \code{X}.
#' 
#' @param deriv A vector composed of \code{0}, \code{1}, and \code{2} of the demanded 
#' functional values / derivatives of the functions in the rows of \code{X}.
#' \code{0} stands for the functional values, \code{1} for the first derivatives, 
#' \code{2} for the second derivatives.
#' 
#' @param range A vector of two elements containing the range of the domain of the functions.
#' By default the range is set to \code{c(0,1)}.
#'
#' @return An array of the functional values and / or the derivatives of \code{X}. Its dimension
#' is \code{m*d*nd}, where \code{m*d} is the dimension of \code{X},
#' and \code{nd} is the demanded number of derivatives at the output, i.e.
#' the length of the vector \code{deriv}.
#' 
#' @examples
#' data(population)
#' A = as.matrix(population[1:20,1:66])
#' B = as.matrix(population[21:50,1:66])
#' derivatives.est(A,deriv=c(0,1))

derivatives.est = function(X,spar=NULL,deriv=c(0,1),range=c(0,1)){
	derd = length(deriv)
      XK = array(dim=c(dim(X),derd))
	t = seq(0,1,length=ncol(X))
	n = nrow(X)	
	rng = range[2]-range[1]
	if (rng<=0) stop("range must be positive")
  	pb = txtProgressBar(min = 0, max = n, style = 3)	
	for (nd in 1:derd){
		if (deriv[nd]==0) XK[,,nd] = X else
		for(i in 1:n){
			if (deriv[nd]==1){
				if(is.null(spar))	XK[i,,nd] = D1ss(t,X[i,])/rng else
				XK[i,,nd] = D1ss(t,X[i,],spl.spar=spar)/rng
				}
			if (deriv[nd]==2){
				if(is.null(spar))	XK[i,,nd] = D2ss(t,X[i,])$y/(rng^2) else
				XK[i,,nd] = D2ss(t,X[i,],spl.spar=spar)$y/(rng^2)
				}
		setTxtProgressBar(pb, i)
		}
	}
	close(pb)
	return(XK)
	}

#' @title Diagnostic plot for first and second order integrated and infimal depths
#'
#' @description
#' Produce the diagnostic plot based on the fist or second order extended integrated / infimal depths.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords integrated depth, infimal depth, shape outlier, diagnostic plot, functional data
#'
#' @details
#' Plots a diagnostic plot of pointwise univariate (or bivariate) depths for all possible points (or couples of points) from the domain of the 
#' functional data. From such a plot it is possible to infer into the first order (or second order) properties of a single function \emph{x} with respect 
#' to the given set of functional data. For \code{order=1}, the integral of the displayed function is the integrated depth of \emph{x}, 
#' the smallest value of the function is the infimal depth of \emph{x}. 
#' For \code{order=2}, the bivariate integral of the displayed surface gives the second order extended 
#' integrated depth of \emph{x}, the infimum of this bivariate function gives the second order infimal depth of \emph{x}. 
#' For details see Nagy et al. (2016) and \code{\link{depth.fd1}}.
#'
#' @param A A single function whose depth is computed, represented by a matrix of its functional values of
#' size \code{1*d}. \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values are evaluated.
#'
#' @param B Random sample functions with respect to which the depth of \code{A} is computed. 
#' \code{B} is represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated. The grid of observation points for the 
#' functions \code{A} and \code{B} must be the same.
#'
#' @param order The order of the depth to be used in the plot, for \code{order=1} produces
#' the plot of univariate marginal depth of \code{A} and \code{nfun} functions from \code{B} 
#' over the domain of the functions. For \code{order=2} produces the bivariate contour plot 
#' of the bivariate depths of \code{A} at couples of points from the domain.
#'
#' @param method The depth that is used in the diagnostic plot. possible values are \code{halfspace} for 
#' the halfspace depth, or \code{simplicial} for the simplicial depth. 
#'
#' @param approx For \code{order=2}, the number of approximations used in the computation of the order extended depth. By default
#' this is set to \code{0}, meaning that the depth is computed at all possible \code{d^2}
#' combinations of the points in the domain. When set to a positive integer, \code{approx}
#' bivariate points are randomly sampled in unit square, and at these points the bivariate depths of the
#' corresponding functional values are computed.
#'
#' @param title The title of the diagnostic plot.
#'
#' @param nfun For \code{order=1}, the number of functions from \code{B} whose coordinate-wise
#' univariate depths of functional values should be displayes with the depth of \code{A}.
#' The depth of \code{A} is displayed in solid red line, the depths of the functions from \code{B}
#' in dashed black.
#'
#' @param plot Logical: should the function by plotted?
#' 
#' @return For \code{order=1} two depth values, and two vectors of pointwise depths:
#'	\itemize{
#'	\item \code{Simpl_FD} the first order integrated depth based on the simplicial depth,
#'	\item \code{Half_FD} the first order integrated depth based on the halfspace depth,
#'	\item \code{Simpl_ID} the first order infimal depth based on the simplicial depth,
#'	\item \code{Half_ID} the first order infimal depth based on the halfspace depth,
#'    \item \code{PSD} the vector of length \code{d} containing the computed 
#'    pointwise univariate simplicial depths used for the computation of \code{Simpl_FD} and \code{Simpl_ID},
#'    \item \code{PHD} the vector of length \code{d} containing the computed 
#'    pointwise univariate halfspace depths used for the computation of \code{Half_FD} and \code{Half_ID}.
#'	}
#'    In addition, the first order integrated / infimal depth diagnostic plot of the function \code{A} with respect to
#'    the random sample given by the functions corresponding to the rows of the matrix \code{B} is produced.
#'
#'    For \code{order=2} four depth values, and two matrices of pointwise depths:
#'	\itemize{
#'	\item \code{Simpl_FD} the second order integrated depth based on the simplicial depth,
#'	\item \code{Half_FD} the second order integrated depth based on the halfspace depth,
#'	\item \code{Simpl_ID} the second order infimal depth based on the simplicial depth,
#'	\item \code{Half_ID} the second order infimal depth based on the halfspace depth,
#'    \item \code{PSD} the matrix of size \code{d*d} containing the computed 
#'    pointwise bivariate simplicial depths used for the computation of \code{Simpl_FD} and \code{Simpl_ID},
#'    \item \code{PHD} the matrix of size \code{d*d} containing the computed 
#'    pointwise bivariate halfspace depths used for the computation of \code{Half_FD} and \code{Half_ID}.
#'	}
#'    In addition, the second order integrated / infimal depth diagnostic plot of the function \code{A} with respect to
#'    the random sample given by the functions corresponding to the rows of the matrix \code{B} is produced.
#' 
#' @references Nagy, S., Gijbels, I. and Hlubinka, D.  (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#'
#' @examples
#' data(population)
#' A1 = as.matrix(population[1,1:66])
#' B = as.matrix(population[21:50,1:66])
#' shape.fd.analysis(A1,B,order=1)
#' shape.fd.analysis(A1,B,order=2,approx=0)
#'
#' @seealso \code{\link{depth.fd1}}
   
shape.fd.analysis = function(A,B,order=1,method=c("halfspace","simplicial"),approx=0,title="",nfun=10,plot=TRUE){
	if (nrow(A)>1) stop("works only for a single function A (matrix 1*d)")
	method = match.arg(method)
	#
	# diagonal
	d = ncol(A)
	HDP = matrix(nrow=nrow(B)+1,ncol=d)
	SDP = HDP
	for(i in 1:d){
		FL = ecdf(B[,i])
		FU = ecdf(-B[,i])
		HDP[,i] = pmin(FL(rbind(B,A)[,i]),FU(-rbind(B,A)[,i]))
		SDP[,i] = FL(rbind(B,A)[,i])*FU(-rbind(B,A)[,i])*nrow(B)^2/choose(nrow(B),2)
		}
	if(order==2){
            DD = DiffDepth(A,B,approx)
		if (method=="halfspace") diag(DD$PHD) = HDP[nrow(B)+1,] else diag(DD$PSD) = SDP[nrow(B)+1,]
            if (plot){
	      	if (method=="halfspace") filled.contour(DD$PHD,main=title,color.palette= function(x)rev(heat.colors(x))) else
	      	filled.contour(DD$PSD,main=title,color.palette = function(x)rev(heat.colors(x)))
			}
	      return(DD)
            }
      if(order==1){
            nfun = min(nrow(B),nfun)
		t = seq(0,1,length=ncol(B))
            DD = list(Simpl_FD = mean(SDP[nrow(B)+1,]), Half_FD = mean(HDP[nrow(B)+1,]), 
                  Simpl_ID = min(SDP[nrow(B)+1,]), Half_ID = min(HDP[nrow(B)+1,]), 
                  PHD = HDP[nrow(B)+1,], PSD = SDP[nrow(B)+1,]) 
	      if (plot) { if (method=="halfspace"){
	           plot(rep(t,nrow(B)+1),HDP,type="n",main=title,ylim = c(0,max(HDP)),ann=FALSE)
	           for(i in 1:nfun) lines(t,HDP[i,],lwd=.75,lty=2)
	           lines(t,HDP[nrow(B)+1,],col=2,lwd=3,lty=1)
	           } else {
	           plot(rep(t,nrow(B)+1),SDP,type="n",main=title,ylim = c(0,max(SDP)),ann=FALSE)
	           for(i in 1:nfun) lines(t,SDP[i,],lwd=.75,lty=2)
	           lines(t,SDP[nrow(B)+1,],col=2,lwd=3,lty=1)
	           }
		}	
	      return(DD)
            }
	}

#' @title Functional depth-based shape outlier detection
#'
#' @description
#' Detects functional outliers of first three orders, based on the order extended integrated depth for functional data.		
#'
#' @author Stanislav Nagy, \email{stanislav.nagy@@kuleuven.be}
#' @keywords integrated depth, shape outlier, functional data
#'
#' @details
#' Using the procedure described in Nagy et al. (2016), the function uses the order extended integrated depths for functions, 
#' see \code{\link{depth.fd1}} and \code{\link{shape.fd.analysis}}, to perform informal functional shape outlier detection. 
#' Outliers of the first order (horizontal shift outliers) are found as the functions with \code{q} \% of smallest (first order)
#' integrated depth values. Second and third order outliers (shape outliers) are found using the extension of the boxplot method
#' for depths as described in the paper Nagy et al. (2016).
#'
#' @param X  Sample functions, represented by a matrix of 
#' size \code{n*d}, where \code{n} is the sample size, and \code{d}
#' is the number equidistant points in the domain of the data at which the functional
#' values of the \code{n} functions are evaluated.
#'
#' @param q The quantile presenting a threshold for the first order outlier detection. Functions with first order integrated depth
#' smaller than the \code{q} quantile of this sample of depths are flagged as potential outliers. If set to \code{NULL}, the
#' the outliers are detected from the first order integrated depth after the log-transformation, as for higher order outliers.
#'
#' @param method The depth that is used in the diagnostic plot. possible values are \code{halfspace} for 
#' the halfspace depth, or \code{simplicial} for the simplicial depth. 
#'
#' @param approx For the computation of the third order integrated depth,
#' the number of approximations used in the computation of the order extended depth. By default
#' this is set to \code{100}, meaning that \code{100}
#' trivariate points are randomly sampled in unit cube, and at these points the trivariate depths of the
#' corresponding functional values. May be set to \code{0} to compute the depth at all possible \code{d^3}
#' combinations of the points in the domain. This choice may result in very slow computation, see also \code{\link{depth.fd1}}. 
#'
#' @param print If the rows of \code{X} are named, \code{print=TRUE} enables a graphical output when the names of the outlying curves
#' are displayed.
#'
#' @param plotpairs If set to \code{TRUE}, the scatter plot of the computed depths for orders \code{1}, \code{2} and \code{3} is
#' is displayed. Here, the depths corresponding to the flagged outliers are plotted in colour.
#'
#' @param max.order Maximal order of shape outlyingness to be computed, can be set to \code{1}, \code{2}, or \code{3}.
#'
#' @param exclude.out Logical variable; exclude the detected lower order outliers in the flagging process? By default \code{TRUE}.
#'
#' @param output Output method, can be set to \code{matrix} for a matrix with logical entries (\code{TRUE} for outliers), or \code{list} for 
#' a list of outliers.
#' 
#' @return A matrix of logical values of size \code{n*4}, where \code{n} is the sample size. In the first three rows indicators of outlyingness
#' of the corresponding functions for orders \code{1}, \code{2} and \code{3} are given, in the fourth row the indicator of outlyingness
#' with respect to the comparison of the first, and third order depths is given. That is, the fist row corresponds to the first order outliers, 
#' the second row to the second order outliers, and the last two rows formally to the third order outliers. Please consult Nagy et al. (2016)
#' to interpret the notion of shape outlyingness.
#' 
#' @references Nagy, S., Gijbels, I. and Hlubinka, D. (2016), Depth-based recognition of shape outlying functions, \emph{Under review}.
#'
#' @examples
#' data(population)
#' X = as.matrix(population[1:30,1:66])
#' shape.fd.outliers(X,print=TRUE,plotpairs=TRUE)
#'
#' @seealso \code{\link{depth.fd1}}, \code{\link{shape.fd.analysis}}

shape.fd.outliers = function(X,q=.05,method=c("halfspace","simplicial"),approx=100,print=FALSE,plotpairs=FALSE,max.order=3,
exclude.out = TRUE, output=c("matrix","list")){
	method = match.arg(method)
	output = match.arg(output)
	if(is.null(rownames(X))){
		print = FALSE
		warning("Rows of X are not named, print is set to FALSE")
		}
	if((max.order>3)|(max.order<1)){
            max.order=3
            warning("Maximal order set to 3")
      }
      if (max.order==3){
      if(approx<50){
		approx=50
		warning("Too small approx value, approximation set to 50")
		}
      }
	n = nrow(X)									# set up the depths
	D = matrix(nrow=max.order,ncol=dim(X)[1])
	if(method=="halfspace"){
		D[1,] = depth.fd1(X,X)$Half_FD
		if(max.order>1) D[2,] = depth.fd1(X,X,order=2)$Half_FD
		if(max.order>2) D[3,] = depth.fd1(X,X,order=3,approx=approx)$Half_FD
		}
	if(method=="simplicial"){
		D[1,] = depth.fd1(X,X)$Simpl_FD
		if(max.order>1) D[2,] = depth.fd1(X,X,order=2)$Simpl_FD
		if(max.order>2) D[3,] = depth.fd1(X,X,order=3,approx=approx)$Simpl_FD
		}
	D = apply(D,1:2,function(x) max(0,x-1/n))
      if(max.order==1) out.rows = 1
      if(max.order==2) out.rows = 2
      if(max.order==3) out.rows = 4
	O = matrix(nrow=out.rows,ncol=ncol(D))					# compute the outliers
	if (print){
		colnames(O) = rownames(X)
		print("first order outliers: ")
		}
	if(!is.null(q)) O[1,]<-D[1,]<quantile(D[1,],q)
      if(is.null(q)){                  # for q null perform the log-transform
		S = -log(D[1,]/1)
		S[D[1,]==0] = Inf
		B = boxplot(S,plot=FALSE)
		O[1,]<-((S>B$stats[5,])|(S==Inf))            
            } 	
	if (print) print(which(O[1,]))
	if(max.order>1) for(i in 1:(max.order-1)){
		if (print){
			if (i==1) print("second order outliers: ")
			if (i==2) print("third order outliers: ")
			}
		S = -log(D[i+1,]/D[i,])
		S[D[i+1,]==0] = Inf
		B = boxplot(S,plot=FALSE)
		if(exclude.out) O[i+1,]<-(((S>B$stats[5,])|(S==Inf))&(O[i,]==FALSE)&(O[1,]==FALSE))
		if(!exclude.out) O[i+1,]<-((S>B$stats[5,])|(S==Inf))
		if (print) print(which(O[i+1,]))
		}
	if(max.order==3){
            if (print) print("1/3 outliers")
      	S = -log(D[3,]/D[1,])
      	S[D[3,]==0] = Inf
      	B = boxplot(S,plot=FALSE)
      	if(exclude.out) O[4,]<-(((S>B$stats[5,])|(S==Inf))&(O[3,]==FALSE)&(O[2,]==FALSE)&(O[1,]==FALSE))
      	if(!exclude.out) O[4,]<-((S>B$stats[5,])|(S==Inf))
      	if (print) print(which(O[4,]))
            }
      if(max.order==1) rownames(O) = "1st"
      if(max.order==2) rownames(O) = c("1st","2nd")
	if(max.order==3){
            rownames(O) = c("1st","2nd","3rd","1/3rd")
	      if (plotpairs) DpairsPlot(D,O)
            }
	if(output=="matrix") return(O)
	if(output=="list") return(apply(O,1,which))
	}

#' @title Internal functions of the depth.fd package
#' @description
#' These functions are not meant to be used at the user-level.
#'
#' @seealso \code{\link{shape.fd.outliers}}

DpairsPlot = function(DB2,O,sp.index=NULL){
# plots a 2x2 scatter of all the pairs of order extended integrated depth values
# for orders 1, 2, and 3 as presented in the Nagy et al. (2016), Figure 5
	cexaxis = 1.3
	cexlab = 1.5
	n = ncol(O)
	col = rep("grey",n)
	colout1 = "olivedrab3"
	colout2 = "navy"
	colout3 = "darkorange"
	colout = c(colout1,colout2,colout3)
	col[O[1,]] = colout1
	col[O[2,]] = colout2
	col[O[3,]] = colout3
	col[O[4,]] = colout3
	pch = rep(16,n)
	pch[O[1,]] = 1
	pch[O[2,]] = 2
	pch[O[3,]] = 18
	pch[O[4,]] = 18
	cx = 1.5
	cex = rep(1,n)
	cex[O[1,]] = cx
	cex[O[2,]] = cx
	cex[O[3,]] = cx
	cex[O[4,]] = cx
	OI = (colSums(O)>0)	# outlier indicator

	op<-par(cex.axis=cexaxis,cex.lab=cexlab,mfrow = c(2, 2),     # 2x2 layout
    		oma = c(3, 3.5, 0.45, 0.45), # two rows of text at the outer left and bottom margin
    		mar = c(1, 1, 0, 0), # space for one row of text at ticks and to separate plots
    		mgp = c(2, 1, 0),    # axis label at 2 rows distance, tick labels at 1 row
    		xpd = NA)            # allow content to protrude into outer margin (and beyond)
	plot(c(0,M<-max(DB2)),c(0,max(DB2)),type="n",ann=FALSE,axes=FALSE,frame=TRUE)
	axis(2)
	title(ylab=expression(FD[2]),line=2.5)
	points(DB2[1,],DB2[2,],col=col,pch=pch,cex=cex,lwd=1.35*cex)
	for(i in 1:n) if(OI[i]) points(DB2[1,i],DB2[2,i],col=col[i],pch=pch[i],cex=cex[i],lwd=1.35*cex[i])
	if (!is.null(sp.index)) points(DB2[1,sp.index],DB2[2,sp.index],pch=16,col="orange")
	segments(0,0,M,M,lty=3,lwd=2)
	plot(c(0,1),c(0,1),type="n",ann=FALSE,axes=FALSE)
	legend(0,.7,c("1st order outliers","2nd order outliers","3rd order outliers"),col=colout,pch=c(1,2,18),cex=cx,bty="n")
	plot(c(0,max(DB2)),c(0,max(DB2)),type="n",ann=FALSE)
	axis(2)
	title(ylab=expression(FD[3]^A),line=2.5)
	axis(1)
	title(xlab=expression(FD[1]),line=3)
	points(DB2[1,],DB2[3,],col=col,pch=pch,cex=cex,lwd=1.35*cex)
	for(i in 1:n) if(OI[i]) points(DB2[1,i],DB2[3,i],col=col[i],pch=pch[i],cex=cex[i],lwd=1.35*cex[i])
	if (!is.null(sp.index)) points(DB2[1,sp.index],DB2[3,sp.index],pch=16,col="orange")	
	segments(0,0,M,M,lty=3,lwd=2)
	plot(c(0,max(DB2)),c(0,max(DB2)),type="n",ann=FALSE,axes=FALSE,frame=TRUE)
	axis(1)
	title(xlab=expression(FD[2]),line=3)
	points(DB2[2,],DB2[3,],col=col,pch=pch,cex=cex,lwd=1.35*cex)
	for(i in 1:n) if(OI[i]) points(DB2[2,i],DB2[3,i],col=col[i],pch=pch[i],cex=cex[i],lwd=1.35*cex[i])
	if (!is.null(sp.index)) points(DB2[2,sp.index],DB2[3,sp.index],pch=16,col="orange")
	segments(0,0,M,M,lty=3,lwd=2)
	par(op)
	}
